Name:		clang
Version:	17.0.6
Release:	6%{?dist}
Summary:	C language family Compiler front-end for LLVM
License:	Apache License v2.0 with LLVM Exceptions
URL:		http://llvm.org
Source0:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}/clang-%{version}.src.tar.xz
Source1:	https://github.com/llvm/llvm-project/releases/download/llvmorg-%{version}/clang-tools-extra-%{version}.src.tar.xz

Patch0001:	0001-Backport-LoongArch-Add-the-support-for-vector.patch
Patch0002:	0002-Backport-LoongArch-improve-the-support-for-compiler-rt-and-bugfix.patch

Patch3000:	0001-clang-tools-extra-Make-test-dependency-on-LLVMHello-.patch
Patch3001:	0010-PATCH-clang-Produce-DWARF4-by-default.patch
Patch3002:	0003-PATCH-clang-Don-t-install-static-libraries.patch
Patch3003:	cfg.patch
Patch3004:	tsa.patch
Patch3005:	0009-disable-recommonmark.patch
Patch5000:	clang-16.0.6-add-OpenCloudOS-support.patch
Patch5001:	0001-LoongArch-Add-support-for-OpenCloudOS-triple.patch

%define maj_ver %(echo %{version} | cut -d. -f1)
%define min_ver %(echo %{version} | cut -d. -f2)
%define patch_ver %(echo %{version} | cut -d. -f3)

BuildRequires:	gcc gcc-c++ cmake ninja-build
BuildRequires:	llvm-devel = %{version}
BuildRequires:	llvm-cmake-utils = %{version}
BuildRequires:	llvm-test = %{version}
BuildRequires:	libxml2-devel perl-generators ncurses-devel
#for %%{_emacs_sitestartdir}
BuildRequires:	emacs
# for manpages
BuildRequires:	python3-sphinx
# for pathfix.py
BuildRequires:	python3-devel
# for py_reproducible_pyc_path
BuildRequires: /usr/bin/marshalparser
%global py_reproducible_pyc_path %{buildroot}%{python3_sitelib}


# for tests
BuildRequires:	python3-lit
BuildRequires:  perl(Digest::MD5) perl(File::Copy) perl(File::Find) perl(File::Path) perl(File::Temp)
BuildRequires:  perl(FindBin) perl(Hash::Util) perl(lib) perl(Term::ANSIColor)
BuildRequires:  perl(Text::ParseWords) perl(Sys::Hostname)

Requires:       %{name}-libs = %{version}-%{release}
# clang requires gcc, clang++ requires libstdc++-devel
Requires:       libstdc++-devel gcc-c++
Provides:       clang(major) = %{maj_ver}

%description
Clang provides a language front-end and tooling infrastructure for languages
in the C language family (C, C++, Objective C/C++, OpenCL, CUDA, and RenderScript)
for the LLVM projects. Both a GCC-compatible compiler driver (clang) and
an MSVC-compatible compiler driver (clang-cl.exe) are provided.

Unlike many other compiler frontends, Clang is useful for a number of things
beyond just compiling code, to intend to be host to a number of different
source-level tools.  One example of this is the Clang Static Analyzer.

Details are https://clang.llvm.org/.

%package libs
Summary:        Runtime library for clang
Requires:       %{name}-resource-filesystem = %{version}
Recommends:     compiler-rt = %{version}
# For compiling with -fopenmp.
Recommends:     libomp-devel = %{version}
Recommends:     libomp = %{version}

%description libs
Runtime library for clang.

%package devel
Summary:        Header files for clang
Requires:       %{name} = %{version}-%{release}
Requires:       %{name}-tools-extra = %{version}-%{release}
Requires:       %{name}-libs = %{version}-%{release}

%description devel
Header files for clang.

%package resource-filesystem
Summary:        Filesystem package for the clang resource directory
Provides:       %{name}-resource-filesystem(major) = %{maj_ver}

%description resource-filesystem
Clang resouce directory: $libdir/clang/$version/

%package analyzer
Summary:        A source code analysis framework
License:        NCSA and MIT
BuildArch:      noarch
Requires:       %{name} = %{version}-%{release}

%description analyzer
Clang Static Analyzer consists of both a source code analysis
framework and a standalone tool that finds bugs in C and Objective-C
programs.

%package tools-extra
Summary:        Extra tools for clang
Requires:       %{name}-libs = %{version}-%{release}
Requires:       emacs-filesystem

%description tools-extra
A set of extra tools built using Clang's tooling API.


%package -n git-clang-format
Summary:	Integration of clang-format for git
Requires:	%{name}-tools-extra = %{version}-%{release}
Requires:	git python3

%description -n git-clang-format
clang-format integration for git.


%package -n python3-clang
Summary:        Python3 bindings for clang
Requires:       %{name}-libs = %{version}-%{release}
Requires:       python3
%description -n python3-clang
Python3 bindings for clang.


%prep
%setup -T -q -b 1 -n clang-tools-extra-%{version}.src
%patch3000 -p2

# failing test case
rm test/clang-tidy/checkers/altera/struct-pack-align.cpp

pathfix.py -i %{__python3} -p -n -k -as \
	clang-tidy/tool/ \
	clang-include-fixer/find-all-symbols/tool/run-find-all-symbols.py

%setup -q -n clang-%{version}.src
%patch0001 -p2
%patch0002 -p2
%patch3001 -p2
%patch3002 -p2
%patch3003 -p2
%patch3004 -p2
%patch3005 -p2
%patch5000 -p1
%patch5001 -p2


pathfix.py -i %{__python3} -p -n -k -as \
	tools/clang-format/ \
	tools/clang-format/git-clang-format \
	utils/hmaptool/hmaptool \
	tools/scan-view/bin/scan-view \
	tools/scan-view/share/Reporter.py \
	tools/scan-view/share/startfile.py \
	tools/scan-build-py/bin/* \
	tools/scan-build-py/libexec/*

# failing test case
rm test/CodeGen/profile-filter.c


%build

# Decrease debuginfo verbosity to reduce memory consumption
%global optflags %(echo %{optflags} | sed 's/-g /-g1 /')
%define _lto_cflags %{nil}

CXXFLAGS="$CXXFLAGS -Wno-address -Wno-nonnull -Wno-maybe-uninitialized"
CFLAGS="$CFLAGS -Wno-address -Wno-nonnull -Wno-maybe-uninitialized"

%cmake  -G Ninja \
	-DLLVM_PARALLEL_LINK_JOBS=1 \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DCMAKE_BUILD_TYPE=RelWithDebInfo \
	-DPYTHON_EXECUTABLE=%{__python3} \
	-DCMAKE_SKIP_RPATH:BOOL=ON \
	-DCLANG_INCLUDE_TESTS:BOOL=ON \
	-DLLVM_EXTERNAL_CLANG_TOOLS_EXTRA_SOURCE_DIR=../clang-tools-extra-%{version}.src \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DLLVM_LIBDIR_SUFFIX=64 \
	-DLLVM_TABLEGEN_EXE:FILEPATH=%{_bindir}/llvm-tblgen \
	-DLLVM_COMMON_CMAKE_UTILS=%{_datadir}/llvm/cmake \
	-DCLANG_ENABLE_ARCMT:BOOL=ON \
	-DCLANG_ENABLE_STATIC_ANALYZER:BOOL=ON \
	-DCLANG_INCLUDE_DOCS:BOOL=ON \
	-DCLANG_PLUGIN_SUPPORT:BOOL=ON \
	-DENABLE_LINKER_BUILD_ID:BOOL=ON \
	-DLLVM_ENABLE_EH=ON \
	-DLLVM_ENABLE_RTTI=ON \
        -DPPC_LINUX_DEFAULT_IEEELONGDOUBLE=ON \
	-DCLANG_LINK_CLANG_DYLIB=ON \
        -DLLVM_ENABLE_NEW_PASS_MANAGER=ON \
        -DLLVM_ENABLE_SPHINX=ON \
	-DSPHINX_WARNINGS_AS_ERRORS=OFF \
        -DLLVM_BUILD_DOCS=ON \
	-DCLANG_BUILD_EXAMPLES:BOOL=OFF \
	-DBUILD_SHARED_LIBS=OFF \
	-DCLANG_REPOSITORY_STRING="%{_vendor} %{version}-%{release}" \
	-DCLANG_DEFAULT_UNWINDLIB=libgcc

%cmake_build


%install
%cmake_install

# don't need static library
rm -rf %{buildroot}%{_libdir}/*.a

# install clang python bindings
mkdir -p %{buildroot}%{python3_sitelib}/clang/
install -p -m644 bindings/python/clang/* %{buildroot}%{python3_sitelib}/clang/
%py_byte_compile %{__python3} %{buildroot}%{python3_sitelib}/clang

# install scanbuild-py
mv %{buildroot}%{_prefix}/%{_lib}/{libear,libscanbuild} %{buildroot}%{python3_sitelib}
%py_byte_compile %{__python3} %{buildroot}%{python3_sitelib}/{libear,libscanbuild}
chmod a+x %{buildroot}%{_datadir}/scan-view/{Reporter.py,startfile.py}

# Move emacs integration files
mkdir -p %{buildroot}%{_emacs_sitestartdir}
for f in clang-format.el clang-rename.el clang-include-fixer.el; do
    mv %{buildroot}{%{_datadir}/clang,%{_emacs_sitestartdir}}/$f
done

# remove editor integrations (bbedit, sublime, emacs, vim)
rm -vf %{buildroot}%{_datadir}/clang/clang-format-bbedit.applescript
rm -vf %{buildroot}%{_datadir}/clang/clang-format-sublime.py*

rm -Rvf %{buildroot}%{_docdir}/Clang/clang/html
rm -Rvf %{buildroot}%{_datadir}/clang/clang-doc-default-stylesheet.css
rm -Rvf %{buildroot}%{_datadir}/clang/index.js
rm -Rvf %{buildroot}%{_includedir}/clang-tidy/
rm -vf %{buildroot}%{_datadir}/clang/bash-autocomplete.sh

# Create Manpage symlinks
ln -s clang.1.gz %{buildroot}%{_mandir}/man1/clang++.1.gz
ln -s clang.1.gz %{buildroot}%{_mandir}/man1/clang-%{maj_ver}.1.gz
ln -s clang.1.gz %{buildroot}%{_mandir}/man1/clang++-%{maj_ver}.1.gz

ln -s clang++ %{buildroot}%{_bindir}/clang++-%{maj_ver}

chmod u-x %{buildroot}%{_mandir}/man1/scan-build.1*


mkdir -p %{buildroot}%{_libdir}/clang/%{maj_ver}/{include,lib,share}/

ln -s %{_datadir}/clang/clang-format-diff.py %{buildroot}%{_bindir}/clang-format-diff

%check
LD_LIBRARY_PATH=%{buildroot}/%{_libdir} %{__ninja} check-all -C %{__cmake_builddir} || :


%files
%license LICENSE.TXT
%{_bindir}/clang
%{_bindir}/clang++
%{_bindir}/clang-%{maj_ver}
%{_bindir}/clang++-%{maj_ver}
%{_bindir}/clang-cl
%{_bindir}/clang-cpp
%{_mandir}/man1/clang.1.gz
%{_mandir}/man1/clang++.1.gz
%{_mandir}/man1/clang-%{maj_ver}.1.gz
%{_mandir}/man1/clang++-%{maj_ver}.1.gz

%files libs
%{_libdir}/clang/%{maj_ver}/include/*
%{_libdir}/*.so.*

%files devel
%{_libdir}/*.so
%{_includedir}/clang/
%{_includedir}/clang-c/
%{_libdir}/cmake/*
%dir %{_datadir}/clang/

%files resource-filesystem
%dir %{_libdir}/clang/

%files analyzer
%{_bindir}/scan-view
%{_bindir}/scan-build
%{_bindir}/analyze-build
%{_bindir}/intercept-build
%{_bindir}/scan-build-py
%{_libexecdir}/ccc-analyzer
%{_libexecdir}/c++-analyzer
%{_libexecdir}/analyze-c++
%{_libexecdir}/analyze-cc
%{_libexecdir}/intercept-c++
%{_libexecdir}/intercept-cc
%{_datadir}/scan-view/
%{_datadir}/scan-build/
%{_mandir}/man1/scan-build.1.*
%{python3_sitelib}/libear
%{python3_sitelib}/libscanbuild


%files tools-extra
%{_bindir}/amdgpu-arch
%{_bindir}/clang-apply-replacements
%{_bindir}/clang-change-namespace
%{_bindir}/clang-check
%{_bindir}/clang-doc
%{_bindir}/clang-extdef-mapping
%{_bindir}/clang-format
%{_bindir}/clang-include-cleaner
%{_bindir}/clang-include-fixer
%{_bindir}/clang-move
%{_bindir}/clang-offload-bundler
%{_bindir}/clang-offload-packager
%{_bindir}/clang-linker-wrapper
%{_bindir}/clang-pseudo
%{_bindir}/clang-query
%{_bindir}/clang-refactor
%{_bindir}/clang-rename
%{_bindir}/clang-reorder-fields
%{_bindir}/clang-repl
%{_bindir}/clang-scan-deps
%{_bindir}/clang-tidy
%{_bindir}/clangd
%{_bindir}/diagtool
%{_bindir}/hmaptool
%{_bindir}/nvptx-arch
%{_bindir}/pp-trace
%{_bindir}/c-index-test
%{_bindir}/find-all-symbols
%{_bindir}/modularize
%{_bindir}/clang-format-diff
%{_mandir}/man1/diagtool.1.gz
%{_emacs_sitestartdir}/clang-format.el
%{_emacs_sitestartdir}/clang-rename.el
%{_emacs_sitestartdir}/clang-include-fixer.el
%{_datadir}/clang/clang-format.py*
%{_datadir}/clang/clang-format-diff.py*
%{_datadir}/clang/clang-include-fixer.py*
%{_datadir}/clang/clang-tidy-diff.py*
%{_bindir}/run-clang-tidy
%{_datadir}/clang/run-find-all-symbols.py*
%{_datadir}/clang/clang-rename.py*

%files -n git-clang-format
%{_bindir}/git-clang-format

%files -n python3-clang
%{python3_sitelib}/clang/


%changelog
* Wed Aug 28 2024 kianli <kianli@tencent.com> - 17.0.6-6
- Avoid recommonmark dependency in OCS builds

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 17.0.6-5
- Rebuilt for loongarch release

* Tue Jul 09 2024 zhanglimin <zhanglimin@loongson.cn> - 17.0.6-4
- Rebuild for clang

* Tue May 28 2024 zhanglimin <zhanglimin@loongson.cn> - 17.0.6-3
- Add support for LoongArch OpenCloudOS triple

* Fri May 17 2024 zhanglimin <zhanglimin@loongson.cn> - 17.0.6-2
- Add the support for vector, compiler-rt and fix some bugs

* Fri Dec 15 2023 Upgrade Robot <upbot@opencloudos.org> - 17.0.6-1
- Upgrade to version 17.0.6

* Tue Sep 19 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 16.0.6-3
- Rebuilt for python 3.11

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 16.0.6-2
- Rebuilt for OpenCloudOS Stream 23.09

* Mon Jul 10 2023 kianli <kianli@tencent.com> - 16.0.6-1
- Upgrade to 16.0.6

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 14.0.5-3
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 14.0.5-2
- Rebuilt for OpenCloudOS Stream 23

* Wed Jun 15 2022 metrechen <metrechen@tencent.com> - 14.0.5-1
- Update to tllvm, based on LLVM 14.0.5

* Fri May 27 2022 rockerzhu <rockerzhu@tencent.com> - 14.0.0-1
- Initail build
